@@ -22,9 +22,7 @@
 package main;
 use Sbuild::Conf;
 use Sbuild::Log qw(open_log close_log);
-use Sbuild::Chroot qw(get_command run_command exec_command
-		      get_apt_command run_apt_command current);
-use Sbuild::Sysconfig;
+use Sbuild::Chroot;
 
 $ENV{'LC_ALL'} = "POSIX";
 $ENV{'SHELL'} = "/bin/sh";
@@ -32,18 +30,24 @@ $ENV{'SHELL'} = "/bin/sh";
 # avoid intermixing of stdout and stderr
 $| = 1;
 
-Sbuild::Conf::init();
 $Sbuild::Conf::verbose++;
-Sbuild::Chroot::init();
 
 package Sbuild::Utility;
 
-use Sbuild::Conf;
-use Sbuild::Chroot qw(begin_session end_session);
-
 use strict;
 use warnings;
 
+use Sbuild::Conf;
+use Sbuild::Chroot;
+use Sbuild::Sysconfig qw($arch);
+
+sub get_dist ($);
+sub setup ($);
+sub cleanup ();
+sub shutdown ($);
+
+my $current_session;
+
 BEGIN {
     use Exporter ();
     our (@ISA, @EXPORT);
@@ -58,7 +62,7 @@ BEGIN {
     $SIG{'PIPE'} = \&shutdown;
 }
 
-sub get_dist {
+sub get_dist ($) {
     my $dist = shift;
 
     $dist = "unstable" if ($dist eq "-u" || $dist eq "u");
@@ -70,7 +74,7 @@ sub get_dist {
     return $dist;
 }
 
-sub setup {
+sub setup ($) {
     my $chroot = shift;
 
     $Sbuild::Conf::nolog = 1;
@@ -78,26 +82,30 @@ sub setup {
 
     $chroot = get_dist($chroot);
 
-    if (!begin_session($chroot)) {
+    # TODO: Allow user to specify arch.
+    my $session = Sbuild::Chroot::new($chroot, undef, undef);
+    $Sbuild::Utility::current_session = $session;
+
+    if (!$session->begin_session()) {
 	print STDERR "Error setting up $chroot chroot\n";
-	return 1;
+	return undef;
     }
 
     if (defined(&main::local_setup)) {
-	return main::local_setup($chroot);
+	return main::local_setup($session);
     }
-    return 0;
+    return $session;
 }
 
-sub cleanup {
+sub cleanup () {
     if (defined(&main::local_cleanup)) {
-	main::local_cleanup();
+	main::local_cleanup($Sbuild::Utility::current_session);
     }
-    end_session();
+    $Sbuild::Utility::current_session->end_session();
     Sbuild::Log::close_log();
 }
 
-sub shutdown {
+sub shutdown ($) {
     cleanup();
     exit 1;
 }
